/************ I N C L U D E   F I L E S *******************************************************/

#include <stdio.h>

#include "Acme Filters.h"
#include "FindOriginal.h"



/**********************************************************************************************
 * Function Name:  FindOriginal
 *
 * Description:    Make the Finder open the folder containing the file linked to an alias.
 *
 * Parameters:     iFile - file spec of a file.
 *
 * Return Value:   None.
 **********************************************************************************************/
void	FindOriginal( FSSpec *iFile )
{
	ProcessSerialNumber	psn = { 0, kNoProcess };
	ProcessInfoRec		info;
	Str32				name;
	FSSpec				file;
	Boolean				found = false;
	
	// get the address of the Finder from the process list
	info.processInfoLength = sizeof( ProcessInfoRec );
	info.processName = name;
	info.processAppSpec = &file;
	while ( !found && (GetNextProcess( &psn ) == noErr) )
	{
		if ( GetProcessInformation( &psn, &info ) == noErr )
		{
			if ( (info.processType == kFinderType) && (info.processSignature == kFinderCreator) )
			{
				found = true;
			}
		}
	}
	
	if ( found )
	{
		AEAddressDesc	address;
		AppleEvent		event;
		AEDesc			folder;
		
		// put the Finder's psn into a descriptor record
		AECreateDesc( typeProcessSerialNumber, &psn, sizeof( ProcessSerialNumber ), &address );
		
		// create an apple event to be sent to the Finder
		AECreateAppleEvent( kCoreEventClass, kAEOpenDocuments, &address, kAutoGenerateReturnID, kAnyTransactionID, &event );
		
		// zap the filename to open the parent directory instead of the file
		iFile->name[0] = 0;
		
		// put the file spec into a descriptor record
		AECreateDesc( typeFSS, iFile, sizeof( FSSpec ), &folder );
		
		// put the descriptor into the apple event
		AEPutParamDesc( &event, keyDirectObject, &folder );
		
		// send the event
		AESend( &event, nil, kAEWaitReply, kAENormalPriority, kAEDefaultTimeout, nil, nil );
		
		AEDisposeDesc( &folder );
		AEDisposeDesc( &event );
		AEDisposeDesc( &address );
	}
}
