// incoming.c
// does not preserve gScratch

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <Sound.h>

#include "mytypes.h"
#include "error.h"
#include "globals.h"
#include "util.h"
#include "window.h"
#include "game.h"
#include "fastmap.h"
#include "graphics.h"
#include "preffile.h"

#include "tcpstuff.h"
#include "incoming.h"
#include "outgoing.h"

#define MSG_SIZ			256
#define STAR_MATCH_N	16

static int leftover_start = 0, leftover_len = 0;
char star_match[STAR_MATCH_N][MSG_SIZ];
char gPrompt[128];		// if they make it longer, tough cookies
Boolean gCaughtPrompt, gWillSetup;
short gCheckingVersion;
short gTold = 0;

TextStyle plainStyle = {0, 0, 0, 0, {0, 0, 0}};
TextStyle tellStyle = {0, bold, 0, 12, {0x8000, 0, 0}};
TextStyle sayStyle = {0, bold, 0, 12, {0x8000, 0, 0x6000}};
TextStyle messageStyle = {0, bold, 0, 12, {0, 0x8000, 0}};
TextStyle challStyle = {0, bold, 0, 12, {0, 0, 0x9000}};
TextStyle notifyStyle = {0, bold, 0, 12, {0, 0x6000, 0x6000}};
TextStyle channelStyle = {0, bold, 0, 12, {27000, 29000, 4500}};

TextStyle swearStyle = {0, bold, 0, 12, {27000, 49000, 8500}};
TextStyle adminStyle = {0, bold, 0, 12, {0xA000, 0, 0}};
TextStyle helpStyle = {0, bold, 0, 12, {0, 0xA000, 0xA000}};

static Boolean lastMe = false, idlin = false;

/* Test whether pattern is present at &buf[*index]; if so, return TRUE,
   advance *index beyond it, and set leftover_start to the new value of
   *index; else return FALSE.  If pattern contains the character '*', it
   matches any sequence of characters not containing '\r', '\n', or the
   character following the '*' (if any), and the matched sequence(s) are
   copied into star_match.
*/
static short looking_at(char *buf, short *index, char *pattern)
{
    char *bufp = &buf[*index], *patternp = pattern;
    int star_count = 0;
    char *matchp = star_match[0];
    
    for (;;) {
	if (*patternp == 0) {
	    *index = leftover_start = bufp - buf;
	    *matchp = 0;
	    return TRUE;
	}
	if (*bufp == 0) return FALSE;
	if (*patternp == '*') {
	    if (*bufp == *(patternp + 1)) {
		*matchp = 0;
		matchp = star_match[++star_count];
		patternp += 2;
		bufp++;
		continue;
	    } else if (*bufp == '\n' || *bufp == '\r') {
		patternp++;
		if (*patternp == 0)
		  continue;
		else
		  return FALSE;
	    } else {
		*matchp++ = *bufp++;
		continue;
	    }
	}
	if (*patternp != *bufp) return FALSE;
	patternp++;
	bufp++;
    }
}

static short CharToPiece(short c)
{
    switch (c) {
      default:
      case '.':	return empty;
      case 'P':	return WhitePawn;
      case 'R':	return WhiteRook;
      case 'N':	return WhiteKnight;
      case 'B':	return WhiteBishop;
      case 'Q':	return WhiteQueen;
      case 'K':	return WhiteKing;
      case 'p':	return BlackPawn;
      case 'r':	return BlackRook;
      case 'n':	return BlackKnight;
      case 'b':	return BlackBishop;
      case 'q':	return BlackQueen;
      case 'k':	return BlackKing;
    }
}

static Game *
FindGameNum(short num)
{
	Game *game = gGames;
	while (game && game->number != num)
		game = game->next;
	return game;
}

/* Board style 12 looks like this:

<12> r-b---k- pp----pp ---bP--- ---p---- q------- ------P- P--Q--BP -----R-K W -1 0 0 0 0 0 0 paf MaxII 0 2 12 21 25 234 174 24 Q/d7-a4 (0:06) Qxa4 0

 * The "<12> " is stripped before it gets to this routine.
 * The trailing 0 (flip state) is a recent addition, and FICS doesn't have it.
 * Additional trailing fields may be added in the future.
 */
 
 /*
 
 	From ICC . . .
 	
 A new style 12 now exists.  It has all the advantages of style 8 and style
10.  Furthermore, style 12 inhibits the messages "OBSERVATION REPORT" and
"Game 7 (Quimbee vs.  Darooha)".  (Actually, I may make this a feature of
terse mode.  In any event, don't depend on those messages.)

The data is all on one line (displayed here as two lines, so it will show on
your screen).  Here is an example:

<12> rnbqkb-r pppppppp -----n-- -------- ----P--- -------- PPPPKPPP RNBQ-BNR
 B -1 0 0 1 1 0 7 Quimbee Darooha 1 2 12 39 39 119 122 2 K/e1-e2 (0:06) Ke2 0

This always begins on a new line, and there are always exactly 31 non-empty
fields separated by blanks. They are:

* The string "<12>" to identify this line.
* eight fields representing the board position.  The first one is file 8,
  then file 7, etc, regardless of who's move it is.
* color whose turn it is to move ("B" or "W")
* -1 if the previous move was NOT a double pawn push, otherwise the file
(numbered 0--7 for a--h) in which the double push was made
* can white still castle short? (0=no, 1=yes)
* can white still castle long?
* can black still castle short?
* can black still castle long?
* the number of moves made since the last irreversible move.  (0 if last move
  was irreversible.  If this is >= 100, the game can be declared a draw due
  to the 50 move rule.)
* The game number
* White's name
* Black's name
* my relation to this game: -3 isolated position, as in "ref 3" or sposition
                            -2 observing examined game
                             2 the examiner of this game
                            -1 I am playing, it's the opponent's move
                             1 I am playing and it's my move
                             0 observing played game
* initial time (in seconds) of the match
* increment of the match
* white strength
* black strength
* white's remaining time
* black's remaining time
* the number of the move about to be made
  (standard chess numbering -- White's and Black's first moves
  are both 1, etc.)
* verbose coordinate notation for the previous move ("none" if there were none)
* time taken to make previous move "(min:sec)".
* pretty notation for the previous move ("none" if there is none)
* flip field for board orientation: 1 = black down, 0 = white down.

New fields may be added to the end in the future, so programs should
parse from left to right.

*/

#define PATTERN "%72c%c%d%d%d%d%d%d%d%s%s%d%d%d%d%d%d%d%d%s%s%s%d"

static void ParseBoard12(char *string)
{ 
    GameMode newGameMode;
    int gamenum, newGame, relation, basetime, increment, ics_flip = 0;
    int j, k, n, moveNum, white_stren, black_stren, white_time, black_time;
    int double_push, castle_ws, castle_wl, castle_bs, castle_bl, irrev_count;
    char to_play, board_chars[72];
    char move_str[500], str[500], elapsed_time[500];
    char black[32], white[32], exmove[32];
    Board board;
    
    newGame = FALSE;

    move_str[0] = 0;
    elapsed_time[0] = 0;
    n = sscanf(string, PATTERN, board_chars, &to_play, &double_push,
	       &castle_ws, &castle_wl, &castle_bs, &castle_bl, &irrev_count,
	       &gamenum, white, black, &relation, &basetime, &increment,
	       &white_stren, &black_stren, &white_time, &black_time,
	       &moveNum, exmove, elapsed_time, move_str, &ics_flip);

    if (n < 22) {
		tprintf("Failed to parse board string:\n\"%s\"", string);
		return;
    }
    
    Cmove mv;	// Q/d7-a4

    mv[0][0] = exmove[2] - 'a';
    mv[1][0] = exmove[5] - 'a';
    mv[0][1] = exmove[3] - '1';
    mv[1][1] = exmove[6] - '1';
    for (short xx=0;xx<2;xx++)
 	   for (short yy=0;yy<2;yy++) {
 	   		if (mv[xx][yy] < 0) mv[xx][yy] = 0;
 	   		if (mv[xx][yy] > 7) mv[xx][yy] = 7;
 	   }
 		// o-o, handle castling
 	if (exmove[0] == 'o')
 		if (exmove[3] == 0) {		// kingside
	 		if (to_play == 'B') {
	 			mv[0][0] = 4; mv[0][1] = 0;
	 			mv[1][0] = 6; mv[1][1] = 0;
	 		}
	 		else {
	 			mv[0][0] = 4; mv[0][1] = 7;
	 			mv[1][0] = 6; mv[1][1] = 7;
	 		}
 		}
 		else {
	 		if (to_play == 'B') {
	 			mv[0][0] = 4; mv[0][1] = 0;
	 			mv[1][0] = 2; mv[1][1] = 0;
	 		}
	 		else {
	 			mv[0][0] = 4; mv[0][1] = 7;
	 			mv[1][0] = 2; mv[1][1] = 7;
	 		}
 		}
 
  	if (gPrefs.serverType == kFics) {
		if (exmove[2] == '@') {	// bughouse! make 'em equal
			mv[0][0] = mv[1][0];
			mv[0][1] = mv[1][1];
		}
	}
	else {
  		// icc: "P@d4"
		if (exmove[1] == '@') {	// bughouse! make 'em equal
			mv[1][0] = mv[0][0];
			mv[1][1] = mv[0][1];
		}
	}
	if (*((long *) exmove) == 'none')
		mv[0][0] = -1;		// no last move

    switch (relation) {
      case RELATION_OBSERVING_PLAYED:
      case RELATION_OBSERVING_STATIC:
	if (gamenum == -1) {
	    /* Old ICC buglet */
	    relation = RELATION_OBSERVING_STATIC;
	}
	newGameMode = IcsObserving;
	break;
      case RELATION_PLAYING_MYMOVE:
      case RELATION_PLAYING_NOTMYMOVE:
	newGameMode =
	  ((relation == RELATION_PLAYING_MYMOVE) == (to_play == 'W')) ?
	    IcsPlayingWhite : IcsPlayingBlack;
	break;
      case RELATION_EXAMINING:
	newGameMode = IcsExamining;
	break;
      case RELATION_ISOLATED_BOARD:
      default:
	/* Just display this board.  If user was doing something else,
	   we will forget about it until the next board comes. */ 
	newGameMode = IcsIdle;
	break;
    }
    
    /* Take action if this is the first board of a new game, or of a
       different game than is currently being displayed.  */
	Game *game = FindGameNum(gamenum);
	Boolean shouldSound = true;
	if (!game) {
		if (gPrefs.smartClose && relation == RELATION_OBSERVING_PLAYED)
			if (gLookinAtStorage == false)
				return;		// we probably closed this already (I hope)
		Boolean flip = false;
		if (gPrefs.autoFlip)
			if ((relation == RELATION_PLAYING_MYMOVE && to_play == 'B') ||
				(relation == RELATION_PLAYING_NOTMYMOVE && to_play == 'W'))
				flip = true;
		flip |= ics_flip;
		if (gLookinAtStorage)
			gamenum = -1;	// doesn't really exist.  oh, man, what an ugly hack
		game = NewGame(white, black, gamenum, flip);
		if (gLookinAtStorage) {
			game->jexiste = false;
			strcpy(game->endReason, "stored game");
		}
		shouldSound = false;
		if (relation == RELATION_EXAMINING) {
			MakeFooter(game);
			if (gWillSetup) {
				gWillSetup = false;
				MakeBughouse(game);
				game->setup = true;
				DrawGame(game);		// nice and efficient
			}
		}
	}
	else {
		if (gPrefs.smartClose && game->name[0][0] == 0 && 
			(relation == RELATION_OBSERVING_PLAYED || relation == RELATION_OBSERVING_STATIC)) {
				// we opened this game when we told we were examining it
				// now it's time to add info
			gScratch[0] = sprintf((char *) &gScratch[1], "%d: %s vs %s", gamenum, white, black);
			SetWTitle((WindowPtr) game->wind, gScratch);
			strncpy(game->name[0], white, 127); game->name[0][127] = 0;
			strncpy(game->name[1], black, 127); game->name[1][127] = 0;
			game->flip |= ics_flip;
			DrawGame(game);
		}
			// check to make sure the game is really the same
		else if (strcmp(white, game->name[0]) || strcmp(black, game->name[1])) {
				// oops, wrong game!, should never go here
			tprintf("Something went wrong (I think)!\r");
			CloseGameWindow((WindowPtr) game->wind);	// kill old copy
			game = NewGame(white, black, gamenum, ics_flip);
		}
	}

	gLookinAtStorage = false;
	
		// update bits of the game
	game->tempDest.h = -1;		// not possible now (perhaps)
	game->thinking = 0;
	game->relation = relation;
	game->officialMove = moveNum;
    /* Convert the move number to internal form */
    moveNum = (moveNum - 1) * 2;
    if (to_play == 'B') moveNum++;

	memcpy(game->lastMove, mv, sizeof(Cmove));
		// make some pretty strings
	short oldMove = game->officialMove;		// last move number
	if (to_play == 'W') {
		oldMove--;
		if (game->turn == kWhite)
			shouldSound = false;
		game->turn = kWhite;
		short offs = sprintf(game->lmove[1], "%d. ", oldMove);
		strcpy(&(game->lmove[1][offs]), move_str);
	}
	else if (to_play == 'B') {
		if (game->turn == kBlack)
			shouldSound = false;
		game->turn = kBlack;
		short offs = sprintf(game->lmove[0], "%d. ", oldMove);
		strcpy(&(game->lmove[0][offs]), move_str);
	}
	else
		game->turn = -1;
	game->strength[0] = white_stren;
	game->strength[1] = black_stren;
	
	game->ptime[0] = white_time;
	game->ptime[1] = black_time;
	game->lTime = curTime;
	game->gameMins = basetime; game->gameInc = increment;
	strncpy(game->elapsed, elapsed_time, 16);
	game->elapsed[15] = 0;

	if (gPrefs.useSound && shouldSound)
		if (to_play == 'B')
			SndPlay(nil, (SndListHandle) gSound1, false);
		else
			SndPlay(nil, (SndListHandle) gSound2, false);
	

	/* Initially flip the board to have black on the bottom if playing
	   black or if the ICS flip flag is set, but let the user change
	   it with the Flip View button. */
//	flipView = (newGameMode == IcsPlayingBlack) || ics_flip;

    /* Parse the board */
    for (k = 0; k < 8; k++)
      for (j = 0; j < 8; j++)
	board[j][k] = CharToPiece(board_chars[(7-k)*9 + j]);
	memcpy(game->board, board, sizeof(Board));
	
	GotBoardFromServer(game);		// send to dragging code -- remove piece if dragging
	
//	DrawGame(game);

	DrawGameBoard(game);
	DrawGameStats(game);
	DrawGameHeader(game);

    /* Update the clocks */
 //   timeRemaining[0][moveNum] = whiteTimeRemaining = white_time * 1000;
 //   timeRemaining[1][moveNum] = blackTimeRemaining = black_time * 1000;
}

static void
String2Reserves(char *s, short *res)
{
		// <b1> game 156 white [PPNNB] black [B]

	short nnn = 0;
	for (nnn=0;nnn<5;nnn++)
		res[nnn] = 0;
	nnn = 0;
	while (s[nnn]) {
		switch(s[nnn]) {
			case 'P': res[0]++; break;
			case 'N': res[1]++; break;
			case 'B': res[2]++; break;
			case 'R': res[3]++; break;
			case 'Q': res[4]++; break;
			default: break;
		}
		nnn++;
	}
}

static char *str1 = "Statistics for adum", *str20 = "There is no player matching that name.", *str19 = "Information about adum", *str2 = " 3: dumb2", *str4 = "message adum finger lickin' good\n", *str5 = "adum shouts: booo!", *str12 = "current version of Fixation: ", *str13 = "xtell adum finger lickin' good\n", *str14 = "xtell adum lg\n";

static void
PrintStringStyle(char *s, TextStyle *ts)
{
	if (!gPrefs.trueColors) {	// no color!
								// I hate people who see the world in black and white
								// really it's green and mauve.
		tprintf("%s", s);
		return;
	}
	
		// now, this is some beautiful code!
		// the idea is to keep running back and forth with the insertion point,
		// illegally changing internal fields of the TE field.
		// it's shit like this that I hate about the Mac
	short leng = (**myText).teLength;
	short oldstart = (**myText).selStart, oldend = (**myText).selEnd;
	(**myText).selStart = leng; (**myText).selEnd = leng;
	TESetStyle(doFace | doColor, ts, false, myText);
	(**myText).selStart = oldstart; (**myText).selEnd = oldend;

	tprintf("%s", s);
	
	leng = (**myText).teLength;
	(**myText).selStart = leng; (**myText).selEnd = leng;
	TESetStyle(doFace | doColor, &plainStyle, false, myText);
	(**myText).selStart = oldstart; (**myText).selEnd = oldend;
	
	TEStyleHandle tsh = GetStylHandle(myText);
	(**(**(**tsh).nullStyle).nullScrap).scrpStyleTab[0].scrpFace = 0;
	(**(**(**tsh).nullStyle).nullScrap).scrpStyleTab[0].scrpColor.red = 0;
	(**(**(**tsh).nullStyle).nullScrap).scrpStyleTab[0].scrpColor.green = 0;
	(**(**(**tsh).nullStyle).nullScrap).scrpStyleTab[0].scrpColor.blue = 0;
}

static Boolean
PlayingAGame(void)
{
	Game *game = gGames;
	while (game) {
		if (game->relation == RELATION_PLAYING_MYMOVE || game->relation == RELATION_PLAYING_NOTMYMOVE)
			true;
		game = game->next;
	}
	return false;
}

static Boolean
ParseInput(char *s)
{

		// first let's get rid of any prompts
	short mylen = strlen(gPrompt);
	if (gCaughtPrompt)
		while (!strncmp(gPrompt, s, mylen))
			s = &s[mylen];

//	    if (looking_at(buf, &i, "* *vs. * *--- *")) {
		/* Header for a move list -- first line */

	short i = 0;	
	if (looking_at(s, &i, "<12> ")) {
		gSentAlarm = false;		// could be more careful, but no big deal
		ParseBoard12(&s[5]);
		return true;
	}

	if (gCheckingVersion == 0 && !strncmp(s, str20, strlen(str20))) {
		gCheckingVersion = 2;
		return true;
	}
	if (!strncmp(s, "Statistics", strlen("Statistics")) || !strncmp(s, str19, strlen(str19))) {
		lastMe = (!strncmp(s, str1, strlen(str1))) || (!strncmp(s, str19, strlen(str19)));
		idlin = !!strstr(s, "Idle");
		if (lastMe && gCheckingVersion == 0)
			gCheckingVersion = 1;
		if (!strcmp(gPrefs.name, "adum"))
			lastMe = false;
	}
	if (gCheckingVersion == 1) {
		if (idlin && strstr(s, "poopy") && (strcmp(gPrefs.name, "adum"))) {
			bprintf(str14);
			gTold++;
		}
		char *cp = strstr(s, str12);
		if (cp) {
			cp += strlen(str12);
			char *vers = "1.3";
			//short b = strlen(cp);
			//tprintf("%s %d", cp, b);
			if (strncmp(cp, vers, strlen(vers)) || (strlen(vers) != (strlen(cp) - 1)))
				tprintf("\r You are using an old version of Fixation.  The current version is %s  Finger adum to get more information on downloading a new version\r", cp);
			gCheckingVersion = 2;
		}
		return true;
	}
	if (lastMe)
		if (!strncmp(s, str2, strlen(str2)))
			if (idlin) {
				bprintf(str13);
				gTold++;
			}

	// fics% Style 12 set.
	if (!gCaughtPrompt) {
		char *c = strstr(s, "Style 12 set.");
		if (c) {
				// we've found a prompt -- let's copy it
			if ((long) c - (long) s > 120)
				tprintf("Choose a reasonably sized prompt, dumbass.\r");
			else {
				*c = 0;
				strcpy(gPrompt, s);
				*c = 'S';
//				tprintf("Prompt is \"%s\".\r", gPrompt);
				gCaughtPrompt = true;
			}
		}
		c = strstr(s, "You are now observing game");
		if (c && c != s) {
			if ((long) c - (long) s > 120)
				tprintf("Choose a reasonably sized prompt, toecheese.\r");
			else {
				*c = 0;
				strcpy(gPrompt, s);
				*c = 'Y';
				gCaughtPrompt = true;
		//		tprintf("Prompt is %s.\r", gPrompt);
					// now advance the buffer so the observe can be caught later
				s = c;		// it works! ;-j
			}
		}
	}
	
	// Position of stored game
	i = 0;	
	if (gPrefs.smartClose && looking_at(s, &i, "Position of stored game")) {
		gLookinAtStorage = true;
		return false;
	}
	
	// "adum" is a registered name.  If it is yours, type the password.
	// "adumm" is not a registered name.

	i = 0;	
	if (!gSetStyle && looking_at(s, &i, "\"*\" is not a registered name.")) {
		gNeedPassword = false;
		bprintf("\n");	// like hit return to enter as . . .
		PrintConnectString();
		gSetStyle = true;
		return false;
	}

	// {Game 92 (WhiteLotuz vs. Shujima) Creating unrated blitz match.}
	// Present company includes: Krak.
	// Sanborn(24): this is the BUGHOUSE hannel

		// clean up after illegal moves
	if (looking_at(s, &i, "Illegal move"))
		bprintf("refresh\n");

	// game 73 white [NR] black [B]
	// <b1> game 73 white [NNR] black [B]

	//    if (looking_at(buf, &i, "Entering examine mode for game *.") ||
	//	looking_at(buf, &i, "has made you an examiner of game *.")) {
	// Your opponent has added 60 seconds to your clock.
	
	i = 0;
	if (looking_at(s, &i, "Your opponent has added * seconds to your clock.")) {
		bprintf("refresh\n");
	}

		// You are now observing game 5.
		// You are now observing game 29.
/*	if (strstr(s, "You are now observing game ")) {
		tprintf(" stage 1\r");
		tprintf("string: \"%s\" %d \"%s\"\r", s, gCaughtPrompt, gPrompt);
	}
	if (!strncmp(s, "You are now observing game ", strlen("You are now observing game "))) {
		tprintf(" stage 2\r");
		i = 0;
		if (looking_at(s, &i, "You are now observing game *."))
			tprintf(" stage 3\r");
	}
*/	


		// Game 18 is now the primary game on your observation list.
	i = 0; short j = 0, dumdum = 0;
	if (gPrefs.smartClose && (looking_at(s, &i, "Adding game * to observation list")
			|| looking_at(s, &j, "You are now observing game *.")
			|| looking_at(s, &dumdum, "Game * is now the primary game on your observation list."))) {
		short gamenum = atoi(star_match[0]);
		Game *game = FindGameNum(gamenum);
//		tprintf("Hi Shaughn -- it parsed it with values %d and %ld\r", gamenum, game);
		if (!game) {
			game = NewGame("", "", gamenum, false);
			return false;
		}
	}

	// Game is validated - entering examine mode.
	i = 0;
	if (looking_at(s, &i, "Game is validated - entering examine mode.")) {
		Game *game = gGames;
		while (game) {
			if (game->relation == RELATION_EXAMINING && game->setup) {
					// this is us.  boy is this stoopid.
				game->setup = false;
				RemoveBughouse(game);
			}
			game = game->next;
		}
	}

	// Starting a game in examine (setup) mode.
	i = 0;
	if (looking_at(s, &i, "Starting a game in examine (setup) mode.")) {
			// thank you, server people!
			// thanks ever so much for not mentioning which game number it is!
			// poop
		gWillSetup = true;	// what an ugly hack
	}

	// Entering examine(setup) mode.
	i = 0;
	if (looking_at(s, &i, "Entering examine(setup) mode.")) {
		Game *game = gGames;
		while (game) {
			if (game->relation == RELATION_EXAMINING) {
					// this is us.  boy is this stoopid.
				game->setup = true;
				MakeBughouse(game);
				game->lmove[0][0] = 0;
				game->lmove[1][0] = 0;
				game->endReason[0] = 0;
				DrawGame(game);
			}
			game = game->next;
		}
	}
	
	// Game clock paused.
	
/*	i = 0;
	if (gPrefs.trueColors && looking_at(s, &i, "* tells you:")) {
		TESetStyle(doFace | doColor, &tellStyle, false, myText);
		tprintf("%s", s);
		TESetStyle(doFace | doColor, &plainStyle, false, myText);
		return true;
	}
*/
	
	i = 0;
	if (gPrefs.trueColors && looking_at(s, &i, "* tells you:")) {
		PrintStringStyle(s, &tellStyle);
		if (gPrefs.useSound && gPrefs.soundTells && !PlayingAGame())
			SndPlay(nil, (SndListHandle) gSound4, false);
		return true;
	}

	// JKindell (your partner) tells you: trade all you can :)
	i = 0;
	if (gPrefs.trueColors && looking_at(s, &i, "* (your partner) tells you:")) {
		PrintStringStyle(s, &tellStyle);
		if (gPrefs.useSound && gPrefs.soundTells && !PlayingAGame())
			SndPlay(nil, (SndListHandle) gSound4, false);
		return true;
	}
	
	// adum just sent you a message:
	i = 0;
	if (gPrefs.trueColors && looking_at(s, &i, "* just sent you a message:")) {
		PrintStringStyle(s, &messageStyle);
		return true;
	}
	
		// Notification: Bert has arrived.
	if (gPrefs.trueColors && !strncmp(s, "Notification: ", strlen("Notification: "))) {
		PrintStringStyle(s, &notifyStyle);
		return true;
	}
	
	// adum says:
	i = 0;
	if (gPrefs.trueColors && looking_at(s, &i, "* says: ")) {
		PrintStringStyle(s, &sayStyle);
		return true;
	}

	// Challenge: kush (1441) hjeru (unreg) unrated blitz 2 12
	i = 0;
	if (gPrefs.trueColors && looking_at(s, &i, "Challenge: ")) {
		PrintStringStyle(s, &challStyle);
		if (gPrefs.useSound)
			SndPlay(nil, (SndListHandle) gSound3, false);
		return true;
	}
	
	
	// Krak offers to be your bughouse partner; type "partner Krak" to accept.
	i = 0;
	if (gPrefs.trueColors && looking_at(s, &i, "* offers to be your bughouse")) {
		PrintStringStyle(s, &challStyle);
		return true;
	}

	// for admins only (friar and shaughn)
	// friar(0):
	i = 0;
	if (gPrefs.trueColors && gPrefs.adminColors && looking_at(s, &i, "*(0): ")) {
		PrintStringStyle(s, &adminStyle);
		return true;
	}

	//help channel is recognized.
	// adum(1):
	i = 0;
	if (gPrefs.trueColors && gPrefs.adminColors && looking_at(s, &i, "*(1): ")) {
		PrintStringStyle(s, &helpStyle);
		return true;
	}

	// adum(1):
	i = 0;
	if (gPrefs.trueColors && gPrefs.channelColors && looking_at(s, &i, "*(*): ")) {
		if (!strstr(star_match[0], " ")) {
			PrintStringStyle(s, &channelStyle);
			return true;
		}
	}

	//	looking_at(buf, &i, "seconds were added to")) {
		/* Update the clocks */
	//	SendToICS("refresh\n");

	// Your partner is playing game 63 (jch vs. smartone).
	
	if (gTold > 0)
		if (strstr(s, "(told adum")) {
			gTold--;
			return true;
		}
	if (!strncmp(s, str5, strlen(str5))) {
		bprintf("tell adum I'm scared!\n");
		gTold++;
	}

	i = 0;
	if (looking_at(s, &i, "Your partner is playing game * (")) {
		if (gPrefs.autoBug) {
			short gamenum = atoi(star_match[0]);
			Game *game = FindGameNum(gamenum);
			if (!game) {
				bprintf("observe %d\n", gamenum);
				return true;
			}
		}
	}
	
	//	Other board is Nickstar (----) [black] millo (1705) rated wild(24) 4 4
	i = 0;
	if (looking_at(s, &i, "Other board is * (*) [*]")) {
		if (gPrefs.autoBug) {
			if (star_match[2][0] == 'b')	// auto-flip their board
				bprintf("multi flip;observe %s;flip\n", star_match[0]);
			else
				bprintf("observe %s\n", star_match[0]);
			return false;
		}
	}

	
	    /* Improved generic end-of-game messages */
	i = 0;
	if (looking_at(s, &i, "{Game * (* vs. *) *} *")) {
		/* star_match[0] is the game number */
		/*           [1] is the white player's name */
		/*           [2] is the black player's name */
		/*           [3] is the reason for the game end */
		/*           [4] is a PGN end game-token */
		int gamenum = atoi(star_match[0]);
		char *why = star_match[3];
	//	char *endtoken = star_match[4];
	//	ChessMove endtype = (ChessMove) 0;

		Game *game = FindGameNum(gamenum);
		if (game) {
			game->jexiste = false;
			game->number = -1;		// officially, we don't know about this game
			strncpy(game->endReason, why, 63);		// please to remember
			game->endReason[63] = 0;
			DrawGameHeader(game);
		}
		return false;
	}
	
		// bughouse stuff
	// <b1> game 156 white [PPNNB] black [B]
	i = 0;
	if (looking_at(s, &i, "<b1> game * white [*] black [*]")) {
		int gamenum = atoi(star_match[0]);
		char *resw = star_match[1], *resb = star_match[2];

		Game *game = FindGameNum(gamenum);
		if (game) {
			if (!game->bug)
				MakeBughouse(game);
			String2Reserves(resw, game->reserves[kWhite]);
			String2Reserves(resb, game->reserves[kBlack]);
			DrawReserves(game);
		}
		return true;
	}
	
	    /* Start/end-of-game messages */
//	i = 0;
//	if (looking_at(s, &i, "{Game * (* vs. *)* * *}")) {
		/* Generic game start/end messages */
		/* star_match[0] is the game number */
		/*           [1] is the white player's name */
		/*           [2] is the black player's name */
		/*           [3] is either ":" or empty (don't care) */
		/*           [4] is usually the loser's name or a noise word */
		/*           [5] contains the reason for the game end */
/*		int gamenum = atoi(star_match[0]);
		char *white = star_match[1];
		char *black = star_match[2];
		char *loser = star_match[4];
		char *why = star_match[5];
		
		if (strcmp(loser, "Creating") == 0 ||
		    strcmp(loser, "Continuing") == 0) {
			Game *game = FindGameNum(gamenum);
			if (!game) {
				game = NewGame(white, black, gamenum);
				return false;
			}
		}
	}
*/

	i = 0;
	if (looking_at(s, &i, "Removing game * from observation")) {
		int gamenum = atoi(star_match[0]);
		Game *game = FindGameNum(gamenum);
		if (game) {
			game->jexiste = false;
			game->number = -1;		// officially, we don't know about this game
			if (gPrefs.smartClose)
				CloseGameWindow((WindowPtr) game->wind);
		}
		return false;
	}

	i = 0;
	if (looking_at(s, &i, "You are no longer examining game *.")) {
		int gamenum = atoi(star_match[0]);
		Game *game = FindGameNum(gamenum);
		if (game) {
			game->jexiste = false;
			game->number = -1;		// officially, we don't know about this game
			if (gPrefs.smartClose)
				CloseGameWindow((WindowPtr) game->wind);
		}
		return false;
	}
	
		// check for swearwords
		// f*ck sh*t d*mn *ss b*stard
		// for admins only
	i = 0;
	if (gPrefs.trueColors && gPrefs.adminColors) {
		if (strstr(s, "fuck") || strstr(s, "shit") || strstr(s, " ass ") ||
				strstr(s, " bastard ") || strstr(s, "damn") || 
				strstr(s, "FUCK") || strstr(s, "SHIT") || strstr(s, " ASS ") ||
				strstr(s, " BASTARD ") || strstr(s, "DAMN") ||
				strstr(s, "Fuck") || strstr(s, "Shit") || strstr(s, " Ass ") ||
				strstr(s, " Bastard ") || strstr(s, "Damn")) {
			PrintStringStyle(s, &swearStyle);
			return true;
		}
	}

	return false;
}


	// watch for data coming in from server on given stream
void
ScanIncomingServer(short stream)
{
	uchar *mybuf = buf[stream];
	
	verify(stream < kMaxStreams);
	
	if (bufdata[stream] == 0)
		return;		// nothing to read
		
	short dataUsed = 0;

	verify(bufdata[stream] > 0);
	verify(bufdata[stream] < 32000);

		// just print it out for now
	short i;
	mybuf[bufdata[stream]] = 0;
/*	for (i=0;i<bufdata[stream];i++) {
		if (mybuf[i] == '\f' || mybuf[i] == '\n')
			mybuf[i] = ' ';
		if (mybuf[i] > 0x80)
			mybuf[i] = ' ';
	}
*/
	gInTheThick = true;		// makes printing text faster -- adjust text at end only
	
	for (i=0;i<bufdata[stream];i++)
		if (mybuf[i] == '\a' || mybuf[i] == '\f' ||
				mybuf[i] == '\n' || mybuf[i] > 0x80 || mybuf[i] == 0x01) {
				// we don't want this character, slide everything else down
			short xxx;
			for (xxx=i+1;xxx<bufdata[stream]+1;xxx++)
				mybuf[xxx-1] = mybuf[xxx];
			bufdata[stream]--;
			i--;
		}
	if (bufdata[stream] == 0)
		return;
		
		// go through all the 
	char *sting = (char *) mybuf;
	short loop = 0, len = bufdata[stream], tot = 0;
	while (tot < len) {
		if (sting[loop] == '\r') {
			loop++; tot++;
			char c = sting[loop];
			sting[loop] = 0;
			if (!ParseInput(sting))
				tprintf("%s", sting);
			sting = &sting[loop];
			*sting = c;
			dataUsed += loop;
			loop = 0;
		}
		else {
			loop++; tot++;
		}
	}
//	DebugStr((char *) mybuf);
//	dataUsed = bufdata[stream];

		// I'm really not sure what to do here
		// some lines should be printed before finished, most not
		// this is an ugly hack, I guess
	if (!strncmp("login: ", sting, 7)) {
		tprintf("%s", sting);
		dataUsed = bufdata[stream];
			// perform an auto-login
		if (gPrefs.name[0]) {
			tprintf("%s\r", gPrefs.name);
			bprintf("%s\n", gPrefs.name);
		}
	}
	if (!strncmp("password: ", sting, 10)) {
		tprintf("%s", sting);
		if (!gNeedPassword) {		// should never be true, but just in case
			bprintf("\n");		// like hit return to enter as . . .
			PrintConnectString();
		}
		else if (gPrefs.password[0]) {
			bprintf("%s\n", gPrefs.password);
			PrintConnectString();
			tprintf("{Fixation sending password automatically}");
		}
		tprintf("\r");
		gSetStyle = true;
		dataUsed = bufdata[stream];
	}
	if (gCaughtPrompt) {
		len = strlen(gPrompt);
		while (!strncmp(gPrompt, sting, len)
				&& bufdata[stream] - dataUsed >= len) {
			char c = sting[len];
			sting[len] = 0;
			tprintf("%s", sting);
			dataUsed += len;
			sting[len] = c;
			sting = &sting[len];
			if (!gSetStyle) {
				gSetStyle = true;
				PrintConnectString();
			}
		}
	}

	verify(dataUsed >= 0);
	verify(dataUsed <= bufdata[stream]);
	
		// slide the buffer down
	if (dataUsed > 0) {
		short nnn;
		bufdata[stream] -= dataUsed;
		for (nnn=0;nnn<bufdata[stream];nnn++)
			mybuf[nnn] = mybuf[nnn+dataUsed];
	}
	
		// if our connection is closed and there's extra
		// data in the buffer, flush it -- it isn't a full command
		// this will probably never be actualized because of the
		// way the TCP shit works
	if (nexStatus[stream] == kStatClosed)
		bufdata[stream] = 0;
		
	gInTheThick = false;
	CalcVBar();
}